@@ -27,13 +27,15 @@ try:
     from openff.evaluator.utils.openmm import openmm_quantity_to_pint
     from openff.evaluator.utils.serialization import TypedJSONDecoder, TypedJSONEncoder
     from openff.evaluator.forcefield import ParameterGradientKey
+    evaluator_import_success = True
 except ImportError:
-    warn_once("Note: Failed to import the optional openff.evaluator package. ")
+    evaluator_import_success = False
 
 try:
     from openff.toolkit.typing.engines import smirnoff
+    toolkit_import_success = True
 except ImportError:
-    warn_once("Note: Failed to import the optional openff-toolkit package. ")
+    toolkit_import_success = False
 
 logger = getLogger(__name__)
 
@@ -166,6 +168,12 @@ class Evaluator_SMIRNOFF(Target):
 
     def __init__(self, options, tgt_opts, forcefield):
 
+        if not evaluator_import_success:
+            warn_once("Note: Failed to import the OpenFF Evaluator - FB Evaluator target will not work. ")
+
+        if not toolkit_import_success:
+            warn_once("Note: Failed to import the OpenFF Toolkit - FB Evaluator target will not work. ")
+
         super(Evaluator_SMIRNOFF, self).__init__(options, tgt_opts, forcefield)
 
         self._options = None  # The options for this target loaded from JSON.
@@ -430,13 +438,13 @@ class Evaluator_SMIRNOFF(Target):
         self.FF.make(mvals)
 
         force_field = smirnoff.ForceField(
-            self.FF.offxml, allow_cosmetic_attributes=True
+            self.FF.offxml, allow_cosmetic_attributes=True, load_plugins=True
         )
 
         # strip out cosmetic attributes
         with tempfile.NamedTemporaryFile(mode="w", suffix=".offxml") as file:
             force_field.to_file(file.name, discard_cosmetic_attributes=True)
-            force_field = smirnoff.ForceField(file.name)
+            force_field = smirnoff.ForceField(file.name, load_plugins=True)
 
         # Determine which gradients (if any) we should be estimating.
         parameter_gradient_keys = []
