@@ -21,13 +21,15 @@ try:
     from openff.recharge.esp.storage import MoleculeESPStore
     from openff.recharge.optimize import ElectricFieldOptimization, ESPOptimization
     from openff.recharge.smirnoff import from_smirnoff
+    recharge_import_success = True
 except ImportError:
-    warn_once("Note: Failed to import the optional openff.recharge package.")
+    recharge_import_success = False
 
 try:
     from openff.toolkit.typing.engines import smirnoff
+    toolkit_import_success = True
 except ImportError:
-    warn_once("Note: Failed to import the optional openff-toolkit package. ")
+    toolkit_import_success = False
 
 logger = getLogger(__name__)
 
@@ -39,6 +41,12 @@ class Recharge_SMIRNOFF(Target):
 
     def __init__(self, options, tgt_opts, forcefield):
 
+        if not recharge_import_success:
+            warn_once("Note: Failed to import the OpenFF Recharge package - FB Recharge_SMIRNOFF target will not work. ")
+
+        if not toolkit_import_success:
+            warn_once("Note: Failed to import the OpenFF Toolkit - FB Recharge_SMIRNOFF target will not work. ")
+
         super(Recharge_SMIRNOFF, self).__init__(options, tgt_opts, forcefield)
 
         # Store a mapping between the FB pval parameters and the expected recharge
@@ -75,7 +83,9 @@ class Recharge_SMIRNOFF(Target):
 
         # Determine which BCC parameters are being optimized.
         force_field = smirnoff.ForceField(
-            os.path.join(self.FF.ffdir, self.FF.offxml), allow_cosmetic_attributes=True
+            os.path.join(self.FF.ffdir, self.FF.offxml),
+            allow_cosmetic_attributes=True,
+            load_plugins=True
         )
 
         bcc_handler = force_field.get_parameter_handler("ChargeIncrementModel")
